﻿using ArtOfTest.WebAii.Design.Execution;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Windows.Forms;
using System.Configuration;
using System.Reflection;
using Telerik.TestStudio.Shared.Communication.Transports;
using System.Runtime.InteropServices;

namespace ReRunFailedTests
{
    public class Class1 : IExecutionExtension
    {
        public string projRoot;
        public string procToRunConfig;
        public bool rerunConfig;
        public bool rerunAsListConfig;
        TestList failedTL;

        //Read from dll config file
        public void ReadSettings()
        {
            ExeConfigurationFileMap map = new ExeConfigurationFileMap();
            map.ExeConfigFilename = Assembly.GetExecutingAssembly().Location + ".config";
            Configuration libConfig = ConfigurationManager.OpenMappedExeConfiguration(map, ConfigurationUserLevel.None);
            AppSettingsSection section = (libConfig.GetSection("appSettings") as AppSettingsSection);
            procToRunConfig = section.Settings["ProcessToRun"].Value;
            bool.TryParse(section.Settings["ReRun"].Value.ToString(), out rerunConfig);
            bool.TryParse(section.Settings["ReRunAsList"].Value.ToString(), out rerunAsListConfig);
        }

        //This function will execute the ArtOfTest.Runner process
        public void StartAOTProcess(String wsTest, string projPathRoot)
        {
            ProcessStartInfo ProcessInfo;
            Process process;
            string arg;
            if (rerunAsListConfig==false)
            {
                arg = "\"test=" + @projPathRoot + "\\" + @wsTest + "\"" + " \"root=" + @projPathRoot + "\"";
            }
            else
            {
                arg = "\"list=" + @projPathRoot + "\\TestLists\\" + @wsTest+"\"";
            }

            ProcessInfo = new ProcessStartInfo();
            ProcessInfo.FileName = @procToRunConfig;
            ProcessInfo.Arguments = @arg;
            ProcessInfo.CreateNoWindow = false;
            ProcessInfo.UseShellExecute = false;
            ProcessInfo.Verb = "runas";
            try
            {
                process = Process.Start(ProcessInfo);
                process.WaitForExit();
                process.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        public System.Data.DataTable OnInitializeDataSource(ExecutionContext executionContext)
        {
            return null;
        }

        public void OnStepFailure(ExecutionContext executionContext, ArtOfTest.WebAii.Design.AutomationStepResult stepResult)
        {
            //do something here
        }

        public void OnAfterTestListCompleted(RunResult result)
        {
            if (!rerunConfig)
            {
                return;
            }

            var failedtestnames = result.TestResults.Where(x => x.Result == ArtOfTest.Common.Design.ResultType.Fail).ToList();
            
            if (rerunAsListConfig)
            {
                foreach (TestResult tr in failedtestnames)
                {
                    failedTL.Tests.Add(new TestInfo(tr.TestId.ToString(), tr.TestPath.ToString()));
                }

                failedTL.SaveToListFile(projRoot);

                StartAOTProcess(failedTL.TestListName.ToString() + ".aiilist", projRoot);
            }
            else
            {
                foreach (TestResult tr in failedtestnames)
                {
                    StartAOTProcess(tr.TestPath.ToString(), @projRoot);
                }
            }
        }

        public void OnBeforeTestListStarted(TestList list)
        {
            if (String.IsNullOrEmpty(procToRunConfig))
            {
                ReadSettings();
            }

            if (!rerunConfig || list.TestListName.Contains("FailedReRun"))
            {
                return;
            }
 
            failedTL = new TestList(list.TestListName.ToString() + "(FailedReRun)", "Automatic", ArtOfTest.WebAii.Design.TestListType.Automated);
            failedTL.ProjectId = list.ProjectId;
            failedTL.Id = list.Id;
            failedTL.Settings = list.Settings;
        }

        public void OnAfterTestCompleted(ExecutionContext executionContext, TestResult result)
        {
            //do something here
        }

        public void OnBeforeTestStarted(ExecutionContext executionContext, ArtOfTest.WebAii.Design.ProjectModel.Test test)
        {
            //get configuration information only for the first test in list
            if (String.IsNullOrEmpty(projRoot)) projRoot = executionContext.DeploymentDirectory.ToString();
        }
    }
}
